﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2015 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Parameters;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;


namespace HeuristicLab.Termination {
  [Item("ThreshholdTerminationCriterion", "Basis class for all termination criteria which specifies some form of threshold.")]
  [StorableClass]
  public abstract class ThresholdTerminationCriterion<T> : TerminationCriterion where T : class, IItem {
    public IValueLookupParameter<T> ThresholdParameter {
      get { return (IValueLookupParameter<T>)Parameters["Threshold"]; }
    }

    [StorableConstructor]
    protected ThresholdTerminationCriterion(bool deserializing) : base(deserializing) { }
    [StorableHook(HookType.AfterDeserialization)]
    private void AfterDeserialization() {
      Initialize();
    }
    protected ThresholdTerminationCriterion(ThresholdTerminationCriterion<T> original, Cloner cloner)
      : base(original, cloner) {
      Initialize();
    }
    protected ThresholdTerminationCriterion()
      : base() {
      Parameters.Add(new ValueLookupParameter<T>("Threshold", "The limit of the termiation criterion."));

      Initialize();
    }

    private void Initialize() {
      ThresholdParameter.ToStringChanged += (s, a) => OnToStringChanged();
    }

    public override string ToString() {
      if (ThresholdParameter.Value == null) return Name;
      else return Name + ": " + ThresholdParameter.Value;
    }
  }
}